==> currents-api-types-0.0.55.tgz: package/payin/parsers.ts <==
import z from 'zod';
import { ReadonlyNonEmptyArray } from 'fp-ts/ReadonlyNonEmptyArray';
import { cast } from '../utils/types';
import { ChannelId } from '../channel/types';
import { UserId } from '../user/types';
import { PAYIN_CHANNEL_API_LITERAL_TYPE, PAYIN_USER_API_LITERAL_TYPE } from './constants';
import { queryOffsetLimitParser } from '../common/parsers/queryParams';
import { currencyAmountParser, currencyParser } from '../currencies/parsers';
import { chargeRoutingContextIdParser } from './route/parsers';
import { payinCompositeKeyIsomorphism } from './isomorphism';
import { PayinCompositeIdSerialized } from './types';

export const SEPARATOR = '::'; // just coincides with UUID_SEPARATOR

// "api src" - looks just like a normal graph src but is not graph src; abstractions are just similar
// (in truth, its because the older api already works like that but we'd like to both move from the graph terminology and keep compatibility)
// [8c2eab11-1e88-41dc-a520-ed22af399f96]
const makeApiIdParser = <
  L extends string,
  IdMap extends {
    [key in L]: Id;
  },
  Id extends string
>(
  literals: ReadonlyNonEmptyArray<L>
) => {
  const makeType = (literal: L) =>
    z.object({
      type: z.literal(literal),
      id: z.string().transform(cast<IdMap[L]>()),
    });
  // to procure type
  const first = makeType(literals[0]);

  return z
    .string()
    .min(SEPARATOR.length + 2 /*id and type*/)
    .transform((uuid, ctx) => {
      const [type, id] = uuid.split(SEPARATOR);
      const next = z.discriminatedUnion('type', [
        first,
        ...(literals.slice(1).map(makeType) as [typeof first, ...typeof first[]]),
      ]);
      const nextR = next.safeParse({ type, id: id as IdMap[L] });
      if (!nextR.success) {
        nextR.error.issues.forEach(ctx.addIssue.bind(ctx));
        return z.NEVER as z.infer<typeof next>;
      } else {
        return nextR.data;
      }
    });
};

const srcLiterals = [PAYIN_USER_API_LITERAL_TYPE] as const;
export const payinApiSrcCompositeParser = makeApiIdParser<
  typeof srcLiterals[number],
  {
    user: UserId;
  },
  UserId
>(srcLiterals);
const dstLiterals = [PAYIN_CHANNEL_API_LITERAL_TYPE] as const;
export const payinApiDstCompositeParser = makeApiIdParser<
  typeof dstLiterals[number],
  {
    channel: ChannelId;
  },
  ChannelId
>(dstLiterals);
export const payinApiSrcDstCompositeParserOptional = z.object({
  src: payinApiSrcCompositeParser.optional(),
  dst: payinApiDstCompositeParser.optional(),
});
export const payinApiSrcDstCompositeParser = z.object({
  src: payinApiSrcCompositeParser,
  dst: payinApiDstCompositeParser,
});

export const payinListQueryParser = queryOffsetLimitParser.merge(payinApiSrcDstCompositeParserOptional);
export const payinAmountParser = currencyAmountParser;
export const payinUnitParser = currencyParser;
export const payinApiAmountParser = z.object({
  amt: payinAmountParser,
});
export const payinAmountUnitParser = z
  .object({
    unit: payinUnitParser,
  })
  .merge(payinApiAmountParser);

export const payinContextParser = z.object({
  contextId: chargeRoutingContextIdParser.optional(),
});

export const payinSerializedKeyParser = z
  .string()
  .transform((s) => payinCompositeKeyIsomorphism.from(s as PayinCompositeIdSerialized));

==> currents-api-types-0.0.58.tgz: package/payin/parsers.ts <==
import z from 'zod';
import { queryOffsetLimitParser } from '../common/parsers/queryParams';
import { currencyAmountParser, currencyParser } from '../currencies/parsers';
import { chargeRoutingContextIdParser } from './route/parsers';
import { payinCompositeKeyIsomorphism } from './isomorphism';
import { PayinCompositeIdSerialized } from './types';
import {
  payinApiDstCompositeParser,
  payinApiDstSerializedParser,
  payinApiSrcCompositeParser,
  payinApiSrcSerializedParser,
} from './parsers.common';

export const payinApiSrcDstCompositeParserOptional = z.object({
  src: payinApiSrcCompositeParser.optional(),
  dst: payinApiDstCompositeParser.optional(),
});

export const payinApiSrcDstSerializedParser = z.object({
  src: payinApiSrcSerializedParser,
  dst: payinApiDstSerializedParser,
});
export const payinApiSrcDstCompositeParser = z.object({
  src: payinApiSrcCompositeParser,
  dst: payinApiDstCompositeParser,
});

export const payinListQueryParser = queryOffsetLimitParser.merge(payinApiSrcDstCompositeParserOptional);
export const payinAmountParser = currencyAmountParser;
export const payinUnitParser = currencyParser;
export const payinApiAmountParser = z.object({
  amt: payinAmountParser,
});
export const payinAmountUnitParser = z
  .object({
    unit: payinUnitParser,
  })
  .merge(payinApiAmountParser);

export const payinContextParser = z.object({
  contextId: chargeRoutingContextIdParser.optional(),
});

export const payinSerializedKeyParser = z
  .string()
  .transform((s) => payinCompositeKeyIsomorphism.from(s as PayinCompositeIdSerialized));
